import os
import subprocess
from jadi import component, interface

from vvv.api.config import SystemConfig, MainConfig
from vvv.api.plugin import Plugin


@interface
class PHPFPMImpl(object):
    default_pidfile = None

    def __init__(self, context):
        pass


@component(PHPFPMImpl)
class PHPFPMDebian(PHPFPMImpl):
    default_pidfile = '/var/run/php5-fpm.pid'
    default_config_file = '/etc/php5/fpm/php-fpm.conf'
    service_name = 'php5-fpm'
    binary = 'php5-fpm'

    @classmethod
    def __verify__(cls):
        return os.path.exists('/etc/debian_version')


@component(PHPFPMImpl)
class PHPFPMCentOS(PHPFPMImpl):
    default_pidfile = '/var/run/php-fpm/php-fpm.pid',
    default_config_file = '/etc/php-fpm.conf'
    service_name = 'php-fpm'
    binary = 'php-fpm'

    @classmethod
    def __verify__(cls):
        return os.path.exists('/etc/rhel-release')


@component(Plugin)
class PHPFPMPlugin(Plugin):
    name = 'php-fpm'

    @classmethod
    def __verify__(cls):
        if not PHPFPMImpl.classes():
            return False
        return subprocess.call(['which', PHPFPMImpl.classes()[0].binary]) == 0

    def add_config_defaults(self, config):
        impl = PHPFPMImpl.any(self.context)

        if isinstance(config, MainConfig):
            config.data.setdefault('websites', [])

            for ws in config.data['websites']:
                for app in ws['apps']:
                    if app['type'] == 'php-fpm':
                        app['params'].setdefault('pm_min', 1)
                        app['params'].setdefault('pm_max', 5)
                        app['params'].setdefault('spare_min', 1)
                        app['params'].setdefault('spare_max', 3)
                        app['params'].setdefault('user', 'www-data')
                        app['params'].setdefault('group', 'www-data')
                        app['params'].setdefault('php_admin_values', {})
                        app['params'].setdefault('php_flags', {})
                        app['params'].setdefault('pm', 'dynamic')
                        app['params'].setdefault('custom_conf', None)

        if isinstance(config, SystemConfig):
            config.data.setdefault('php-fpm', {})
            config.data['php-fpm'].setdefault('config_file', impl.default_config_file)
            config.data['php-fpm'].setdefault('pidfile', impl.default_pidfile)



import subprocess
from jadi import component

from vvv.api.app import AppType
from vvv.api.config import SystemConfig
from vvv.api.configurable import Configurable
from vvv.api.check import Check
from vvv.api.restartable import Restartable
from vvv.api.template import Template

from .plugin import PHPFPMImpl


@component(Check)
class PHPFPMServiceCheck(Check):
    name = 'php-fpm is running'

    def run(self):
        return subprocess.call(
            ['service', PHPFPMImpl.any(self.context).service_name, 'status']
        ) == 0


@component(AppType)
class PHPFPMAppType(AppType):
    name = 'php-fpm'

    def get_access_type(self, website_config, app_config):
        return 'fcgi'

    def get_access_params(self, website_config, app_config):
        return {
            'url': 'unix:/var/run/veb-app-%s-%s.sock' % (website_config['name'], app_config['name'])
        }


@component(Configurable)
class PHPFPM(Configurable):
    name = 'php-fpm'

    def configure(self):
        open(SystemConfig.get(self.context).data['php-fpm']['config_file'], 'w').write(
            Template.by_name(self.context, 'fpm.conf').render()
        )
        PHPFPMRestartable.get(self.context).schedule_restart()


@component(Restartable)
class PHPFPMRestartable(Restartable):
    name = 'php-fpm'

    def do_restart(self):
        subprocess.check_call(['service', PHPFPMImpl.any(self.context).service_name, 'restart'])



# encoding: utf8
import logging
import os
import subprocess
import sys
import tempfile
import yaml
from jadi import component

from vvv.api.command import Command, CommandArgumentError
from vvv.api.config import MainConfig
from vvv.api.check import Check, CheckFailure
from vvv.api.configurable import Configurable
from vvv.api.restartable import Restartable


@component(Command)
class CheckCommand(Command):
    name = 'check'
    usage = '''
check - run configuration checks
'''

    def run(self):
        failed = 0
        for check in Check.all(self.context):
            instances = check.get_instances()
            for instance in instances:
                try:
                    result = check.run(*instance)
                    message = None
                except CheckFailure as e:
                    result = False
                    message = str(e)

                if result:
                    logging.info('✓ %s' % check.get_name(*instance))
                else:
                    logging.error('✗ %s - failed' % check.get_name(*instance))
                    if message:
                        logging.error(message)
                    failed += 1

        if failed > 0:
            logging.error('------------------')
            logging.error('%i check(s) failed', failed)
            logging.error('------------------')
        else:
            logging.info('-------------------')
            logging.info('✓ All checks passed')
            logging.info('-------------------')


@component(Command)
class ApplyCommand(Command):
    name = 'apply'
    usage = '''
apply - re-apply current configuration
'''

    def run(self):
        for configurable in Configurable.all(self.context):
            logging.info('Configuring %s', configurable.name)
            configurable.configure()

        for restartable in Restartable.all(self.context):
            if restartable.scheduled:
                restartable.restart()

        logging.info('Configuration applied')


@component(Command)
class AddWebsiteCommand(Command):
    name = 'add'
    usage = '''
add <short-name> - creates a new website
'''

    def consume_arguments(self, argv):
        if not argv:
            raise CommandArgumentError('short-name is required')
        self.website_name = argv.pop(0)

    def run(self):
        ws = {
            'name': self.website_name,
        }

        cfg = MainConfig.get(self.context)
        cfg.data['websites'].append(ws)
        cfg.save()

        logging.info('Website %s added', self.website_name)


@component(Command)
class ListWebsitesCommand(Command):
    name = 'list'
    usage = '''
list - lists websites
'''

    def run(self):
        cfg = MainConfig.get(self.context)
        for ws in cfg.data['websites']:
            print '- %s' % ws['name']


@component(Command)
class EditWebsiteCommand(Command):
    name = 'edit'
    usage = '''
edit <short-name> - launches config editor for a website
'''

    def consume_arguments(self, argv):
        if not argv:
            raise CommandArgumentError('short-name is required')
        self.website_name = argv.pop(0)

    def run(self):
        cfg = MainConfig.get(self.context)
        for ws in cfg.data['websites']:
            if ws['name'] == self.website_name:
                self.website = ws
                break
        else:
            logging.critical('Website "%s" not found', self.website_name)
            return

        tmp = tempfile.NamedTemporaryFile(delete=False)
        tmp.write(yaml.safe_dump(
            ws,
            default_flow_style=False,
            encoding='utf-8',
            allow_unicode=True,
        ))
        tmp.close()

        editor = os.environ.get('EDITOR', 'nano' if os.path.exists('/usr/bin/nano') else 'vi')

        while True:
            if subprocess.call([editor, tmp.name]) == 0:
                try:
                    new_config = yaml.load(open(tmp.name))
                except Exception as e:
                    logging.critical('Config content is invalid: %s', str(e))
                    logging.info('File is retained in %s', tmp.name)
                    logging.info('Press Enter to continue editing, Ctrl-C to abort')
                    try:
                        raw_input()
                    except KeyboardInterrupt:
                        logging.critical('Aborted')
                        return
                    continue

                ws.update(new_config)
                os.unlink(tmp.name)
                cfg.save()
                logging.info('Updated configuration for %s', self.website_name)
                break
            else:
                logging.critical('Editor exited with an erroneous exitcode')
                logging.info('File is retained in %s', tmp.name)


@component(Command)
class ImportWebsiteCommand(Command):
    name = 'import'
    usage = '''
import - import a YAML website config from stdin
'''

    def run(self):
        cfg = MainConfig.get(self.context)

        try:
            website = yaml.load(sys.stdin)
        except Exception as e:
            raise Exception('Config content is invalid: %s', str(e))

        for ws in cfg.data['websites']:
            if ws['name'] == website['name']:
                ws.update(website)
                break
        else:
            cfg.data['websites'].append(website)

        cfg.save()
        logging.info('Updated configuration for %s', website['name'])


@component(Command)
class ExportWebsiteCommand(Command):
    name = 'export'
    usage = '''
export <short-name> - exports YAML config for a website
'''

    def consume_arguments(self, argv):
        if not argv:
            raise CommandArgumentError('short-name is required')
        self.website_name = argv.pop(0)

    def run(self):
        cfg = MainConfig.get(self.context)
        for ws in cfg.data['websites']:
            if ws['name'] == self.website_name:
                return yaml.safe_dump(
                    ws,
                    default_flow_style=False,
                    encoding='utf-8',
                    allow_unicode=True,
                )
        else:
            raise Exception('Website "%s" not found', self.website_name)



import os
from jadi import component, interface

from vvv.api.config import SystemConfig, MainConfig
from vvv.api.plugin import Plugin


@interface
class SupervisorImpl(object):
    default_pidfile = None

    def __init__(self, context):
        pass


@component(SupervisorImpl)
class SupervisorDebian(SupervisorImpl):
    default_config_file = '/etc/supervisor/supervisord.conf'
    service_name = 'supervisor'

    @classmethod
    def __verify__(cls):
        return os.path.exists('/etc/debian_version')


@component(SupervisorImpl)
class SupervisorCentOS(SupervisorImpl):
    default_config_file = '/etc/supervisord.conf'
    service_name = 'supervisord'

    @classmethod
    def __verify__(cls):
        return os.path.exists('/etc/redhat-release')


@component(Plugin)
class SupervisorPlugin(Plugin):
    name = 'supervisor'

    def add_config_defaults(self, config):
        impl = SupervisorImpl.any(self.context)

        if isinstance(config, MainConfig):
            config.data.setdefault('websites', [])

            for ws in config.data['websites']:
                for app in ws['apps']:
                    if app['type'] == 'generic':
                        app['params'].setdefault('user', 'root')
                        app['params'].setdefault('environment', '')
                        app['params'].setdefault('directory', None)
                        app['params'].setdefault('command', None)

        if isinstance(config, SystemConfig):
            config.data.setdefault('supervisor', {})
            config.data['supervisor'].setdefault('config_file', impl.default_config_file)



from jadi import component

from vvv.api.template import Template

@component(Template)
class ConfigFileTemplate(Template):
    name = 'fpm.conf'
    data = """
[global]
pid = ${system_config['php-fpm']['pidfile']}
error_log = ${system_config['log_dir']}/php-fpm.log

[global-pool]
user = ${system_config['nginx']['user']}
group = ${system_config['nginx']['user']}

listen = /var/run/php-fpm.sock
listen.owner = ${system_config['nginx']['user']}
listen.group = ${system_config['nginx']['user']}
listen.mode = 0660

pm = dynamic
pm.start_servers = 1
pm.max_children = 5
pm.min_spare_servers = 1
pm.max_spare_servers = 5

% for website in enabled_websites:
    % for app in website['apps']:
        % if app['type'] == 'php-fpm':

[veb-app-${website['name']}-${app['name']}]
user = ${app['params']['user']}
group = ${app['params']['group']}

listen = /var/run/veb-app-${website['name']}-${app['name']}.sock
listen.owner = ${system_config['nginx']['user']}
listen.group = ${system_config['nginx']['user']}
listen.mode = 0660

pm = ${app['params']['pm']}
pm.start_servers = ${app['params']['pm_min']}
pm.max_children = ${app['params']['pm_max']}
pm.min_spare_servers = ${app['params']['spare_min']}
pm.max_spare_servers = ${app['params']['spare_max']}

% for k in app['params']['php_admin_values']:
php_admin_value[${k}] = ${app['params']['php_admin_values'][k]}
% endfor

% for k in app['params']['php_flags']:
php_flag[${k}] = ${app['params']['php_flags'][k]}
% endfor

${app['params']['custom_conf'] or ''}

        % endif
    % endfor
% endfor
    """



from jadi import component

from vvv.api.template import Template

@component(Template)
class ConfigFileTemplate(Template):
    name = 'nginx.conf'
    data = """
#AUTOMATICALLY GENERATED - DO NO EDIT!

user ${system_config['nginx']['user']} ${system_config['nginx']['user']};
pid /var/run/nginx.pid;
worker_processes ${system_config['nginx']['workers']};
worker_rlimit_nofile 100000;

events {
    worker_connections  4096;
    include /etc/nginx.custom.events.d/*.conf;
}

http {
    default_type application/octet-stream;

    access_log off;
    error_log  ${system_config['log_dir']}/nginx/error.log crit;

    sendfile on;
    tcp_nopush on;

    keepalive_timeout 20;
    client_header_timeout 20;
    client_body_timeout 20;
    reset_timedout_connection on;
    send_timeout 20;

    types_hash_max_size 2048;

    gzip on;
    gzip_disable "msie6";
    gzip_proxied any;
    gzip_min_length 256;
    gzip_comp_level 4;
    gzip_types text/plain text/css application/json application/x-javascript text/xml application/xml application/xml+rss text/javascript application/javascript text/x-js;

    server_names_hash_bucket_size 128;

    include mime.conf;
    charset UTF-8;

    open_file_cache max=100000 inactive=20s;
    open_file_cache_valid 30s;
    open_file_cache_min_uses 2;
    open_file_cache_errors on;

    server_tokens off;

    include proxy.conf;
    include fcgi.conf;

    include conf.d/*.conf;
    include /etc/nginx.custom.d/*.conf;
}

include /etc/nginx.custom.global.d/*.conf;

    """


@component(Template)
class ProxyConfigFileTemplate(Template):
    name = 'nginx.proxy.conf'
    data = """
proxy_redirect          off;
proxy_set_header        Host            $host;
proxy_set_header        X-Real-IP       $remote_addr;
proxy_set_header        X-Forwarded-For $proxy_add_x_forwarded_for;
client_body_buffer_size 128k;
proxy_connect_timeout   90;
proxy_send_timeout      90;
proxy_read_timeout      90;
proxy_buffers           32 4k;
"""


@component(Template)
class FCGIConfigFileTemplate(Template):
    name = 'nginx.fcgi.conf'
    data = """
fastcgi_param   QUERY_STRING            $query_string;
fastcgi_param   REQUEST_METHOD          $request_method;
fastcgi_param   CONTENT_TYPE            $content_type;
fastcgi_param   CONTENT_LENGTH          $content_length;

fastcgi_param   SCRIPT_FILENAME         $document_root$fastcgi_script_name;
fastcgi_param   SCRIPT_NAME             $fastcgi_script_name;
fastcgi_param   PATH_INFO               $fastcgi_path_info;
fastcgi_param   REQUEST_URI             $request_uri;
fastcgi_param   DOCUMENT_URI            $document_uri;
fastcgi_param   DOCUMENT_ROOT           $document_root;
fastcgi_param   SERVER_PROTOCOL         $server_protocol;

fastcgi_param   GATEWAY_INTERFACE       CGI/1.1;
fastcgi_param   SERVER_SOFTWARE         nginx/$nginx_version;

fastcgi_param   REMOTE_ADDR             $remote_addr;
fastcgi_param   REMOTE_PORT             $remote_port;
fastcgi_param   SERVER_ADDR             $server_addr;
fastcgi_param   SERVER_PORT             $server_port;
fastcgi_param   SERVER_NAME             $server_name;

fastcgi_param   HTTPS                   $https;

fastcgi_param   REDIRECT_STATUS         200;
"""


@component(Template)
class MIMEConfigFileTemplate(Template):
    name = 'nginx.mime.conf'
    data = """
types {
    text/html                             html htm shtml;
    text/css                              css;
    text/xml                              xml rss;
    image/gif                             gif;
    image/jpeg                            jpeg jpg;
    application/x-javascript              js;
    text/plain                            txt;
    text/x-component                      htc;
    text/mathml                           mml;
    image/png                             png;
    image/svg+xml                         svg svgz;
    image/x-icon                          ico;
    image/x-jng                           jng;
    image/vnd.wap.wbmp                    wbmp;
    application/java-archive              jar war ear;
    application/mac-binhex40              hqx;
    application/pdf                       pdf;
    application/x-cocoa                   cco;
    application/x-java-archive-diff       jardiff;
    application/x-java-jnlp-file          jnlp;
    application/x-makeself                run;
    application/x-perl                    pl pm;
    application/x-pilot                   prc pdb;
    application/x-rar-compressed          rar;
    application/x-redhat-package-manager  rpm;
    application/x-sea                     sea;
    application/x-shockwave-flash         swf;
    application/x-stuffit                 sit;
    application/x-tcl                     tcl tk;
    application/x-x509-ca-cert            der pem crt;
    application/x-xpinstall               xpi;
    application/zip                       zip;
    application/octet-stream              deb;
    application/octet-stream              bin exe dll;
    application/octet-stream              dmg;
    application/octet-stream              eot;
    application/octet-stream              iso img;
    application/octet-stream              msi msp msm;
    audio/mpeg                            mp3;
    audio/ogg                             oga ogg;
    audio/wav                             wav;
    audio/x-realaudio                     ra;
    video/mp4                             mp4;
    video/mpeg                            mpeg mpg;
    video/ogg                             ogv;
    video/quicktime                       mov;
    video/webm                            webm;
    video/x-flv                           flv;
    video/x-msvideo                       avi;
    video/x-ms-wmv                        wmv;
    video/x-ms-asf                        asx asf;
    video/x-mng                           mng;
}
    """

@component(Template)
class WebsiteConfigFileTemplate(Template):
    name = 'nginx.website.conf'
    data = """
#AUTOMATICALLY GENERATED - DO NO EDIT!

${website['custom_conf_toplevel'] or ''}

server {
    % if website['domains']:
    server_name
    % for domain in website['domains']:
        ${domain['domain']}
    % endfor
        ;
    % endif

    % for port in website['ports']:
    listen ${port['host']}:${port['port']}
        % if port['ssl']:
            ssl
        % endif
        % if port['spdy']:
            spdy
        % endif
        % if port['default']:
            default_server
        % endif
        ;
    % endfor

    % if website['ssl_cert_path']:
    ssl_certificate ${website['ssl_cert_path']};
    ssl_certificate_key ${website['ssl_key_path']};
    % endif

    access_log ${system_config['log_dir']}/${website['name']}/nginx.access.log;
    error_log  ${system_config['log_dir']}/${website['name']}/nginx.error.log;

    % if website['root']:
    root ${website['root']};
    % endif

    index index.html index.htm index.php;

    % if not website['maintenance_mode']:
    ${website['custom_conf'] or ''}

    % for location in location_info:
    location ${location['pattern']} ${
        {
            'exact': '',
            'regex': '~',
            'force-regex': '^~',
        }[location['match']]
    } {
        ${location['custom_conf'] or ''}

        % if not location['custom_conf_override']:
            %if location['path_append_pattern']:
                root ${location['path']};
            % endif
            %if not location['path_append_pattern']:
                alias ${location['path']};
            % endif

            % if location['type'] == 'static':
                %if location['params']['autoindex']:
                    autoindex on;
                % endif
            % endif

            % if location['type'] == 'proxy':
                proxy_pass ${location['params']['url']};
                proxy_set_header Host $http_host;
                proxy_set_header X-Real-IP $remote_addr;
                proxy_set_header X-Forwarded-Proto $scheme;
                proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
                proxy_set_header X-Scheme $scheme;
                proxy_set_header Upgrade $http_upgrade;
                proxy_set_header Connection $http_connection;
                proxy_set_header HTTPS $https;
                proxy_http_version 1.1;
            % endif

            % if location['type'] == 'fcgi':
                fastcgi_index index.php;
                include fcgi.conf;
                fastcgi_pass ${location['params']['url']};
                fastcgi_param SCRIPT_FILENAME $document_root$fastcgi_script_name;
            % endif

        % endif
    }
    
    % endfor

    % endif

    % if website['maintenance_mode']:
    location / {
        return 503;
        error_page 503 @maintenance;
    }

    location @maintenance {
        root /var/lib/ajenti/plugins/vh/extras; # TODO
        rewrite ^(.*)$ /maintenance.html break;
    }
    % endif
}

"""


TEMPLATE_LOCATION_CONTENT_PHP_FCGI = """
        fastcgi_index index.php;
        include fcgi.conf;
        fastcgi_pass unix:/var/run/ajenti-v-php-fcgi-%(id)s.sock;
        fastcgi_param SCRIPT_FILENAME $document_root$fastcgi_script_name;
"""

TEMPLATE_LOCATION_CONTENT_PYTHON_WSGI = """
        proxy_pass http://unix:/var/run/ajenti-v-gunicorn-%(id)s.sock;
        proxy_set_header Host $http_host;
        proxy_set_header X-Real-IP $remote_addr;
        proxy_set_header X-Forwarded-Proto $scheme;
        proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
"""

TEMPLATE_LOCATION_CONTENT_RUBY_UNICORN = """
        proxy_pass http://unix:/var/run/ajenti-v-unicorn-%(id)s.sock;
        proxy_set_header Host $http_host;
        proxy_set_header X-Real-IP $remote_addr;
        proxy_set_header X-Forwarded-Proto $scheme;
        proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
"""

TEMPLATE_LOCATION_CONTENT_RUBY_PUMA = """
        proxy_pass http://unix:/var/run/ajenti-v-puma-%(id)s.sock;
        proxy_set_header Host $http_host;
        proxy_set_header X-Real-IP $remote_addr;
        proxy_set_header X-Forwarded-Proto $scheme;
        proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
"""


TEMPLATE_LOCATION_CONTENT_NODEJS = """
        proxy_pass http://127.0.0.1:%(port)i;
        proxy_set_header Host $http_host;
        proxy_set_header X-Real-IP $remote_addr;
        proxy_set_header X-Forwarded-Proto $scheme;
        proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
"""



import logging

from jadi import interface


@interface
class Restartable(object):
    name = None

    def __init__(self, context):
        self.context = context
        self.scheduled = False

    def schedule_restart(self):
        logging.debug('Scheduling restart of %s', self.name)
        self.scheduled = True

    def restart(self):
        logging.info('Restarting %s', self.name)
        self.do_restart()
        self.scheduled = False

    def do_restart(self):
        raise NotImplementedError



import logging
from jadi import component

from vvv.api.command import Command, CommandArgumentError


@component(Command)
class HelpCommand(Command):
    name = 'help'
    usage = '''
help [command-name] - get help for a command
'''

    def consume_arguments(self, argv):
        if not argv:
            raise CommandArgumentError('command-name is required')
        self.command_name = argv.pop(0)

    def run(self):
        command = Command.by_name(self.context, self.command_name)
        if not command:
            logging.critical('Unknown command %s', self.command_name)
            return

        if not command.usage:
            logging.critical('Command %s has no usage information', self.command_name)
            return

        return command.usage



from jadi import component

from vvv.api.config import MainConfig
from vvv.api.plugin import Plugin


@component(Plugin)
class NodeJSPlugin(Plugin):
    name = 'nodejs'

    def add_config_defaults(self, config):
        if isinstance(config, MainConfig):
            config.data.setdefault('websites', [])

            for ws in config.data['websites']:
                if ws['enabled']:
                    for app in ws['apps']:
                        if app['type'] == 'nodejs':
                            app['params'].setdefault('script', 'app.js')
                            app['params'].setdefault('node_binary', 'node')
                            app['params'].setdefault('user', 'root')
                            app['params'].setdefault('port', 8000)
                            app['params'].setdefault('environment', None)



import os
import pwd
import grp


def ensure_directory(path, uid=None, gid=None, mode=None):
    if not os.path.exists(path):
        os.makedirs(path)
    if uid is not None:
        if isinstance(uid, basestring):
            if gid is None:
                gid = uid
            uid = pwd.getpwnam(uid).pw_uid
        if gid is None:
            gid = uid
        if isinstance(gid, basestring):
            gid = grp.getgrnam(gid).gr_gid
        os.chown(path, uid, gid)
    if mode:
        os.chmod(path, mode)


def absolute_path(path, root):
    if path[0] != '/':
        return os.path.join(root, path)
    return path



from jadi import component

from vvv.api.command import Command
from vvv.api.plugin import Plugin


@component(Command)
class PluginsCommand(Command):
    name = 'plugins'
    usage = '''
plugins - lists available plugins
'''

    def run(self):
        plugins = sorted(Plugin.classes(), key=lambda x: x.name)
        for plugin in plugins:
            print plugin.name



from jadi import component

from vvv.api.config import MainConfig
from vvv.api.plugin import Plugin


@component(Plugin)
class PythonPlugin(Plugin):
    name = 'python'

    def add_config_defaults(self, config):
        if isinstance(config, MainConfig):
            config.data.setdefault('websites', [])

            for ws in config.data['websites']:
                for app in ws['apps']:
                    if app['type'] == 'python-wsgi':
                        app['params'].setdefault('module', 'wsgi')
                        app['params'].setdefault('virtualenv', None)
                        app['params'].setdefault('user', 'root')
                        app['params'].setdefault('workers', None)
                        app['params'].setdefault('environment', None)
                        app['params'].setdefault('custom_conf', None)



import logging
import os
import shutil
import subprocess
from jadi import component

from vvv.api.app import AppType
from vvv.api.config import MainConfig, SystemConfig
from vvv.api.configurable import Configurable
from vvv.api.check import Check, CheckFailure
from vvv.api.restartable import Restartable
from vvv.api.template import Template
from vvv.api.util import ensure_directory, absolute_path


@component(Check)
class NginxConfigCheck(Check):
    name = 'nginx configuration passes test'

    def run(self):
        p = subprocess.Popen(['nginx', '-t'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        o, e = p.communicate()
        if p.returncode != 0:
            raise CheckFailure(o + e)
        return True


@component(Check)
class NginxServiceCheck(Check):
    name = 'nginx is running'

    def run(self):
        return subprocess.call(['service', 'nginx', 'status']) == 0


@component(Configurable)
class NginxWebserver(Configurable):
    name = 'nginx'

    def __generate_website_config(self, website):
        location_info = []
        for location in website['locations']:
            if location['type'] in ['static', 'proxy', 'fcgi']:
                location_info.append(location)
            elif location['type'] == 'app':
                for app in website['apps']:
                    if app['name'] == location['params']['app']:
                        break
                else:
                    logging.warn('Skipping unknown app "%s"', location['params']['app'])
                    continue

                app_type = AppType.by_name(self.context, app['type'])
                if not app_type:
                    logging.warn('Skipping unknown app type "%s"', app['type'])
                    continue
                new_location = location.copy()
                new_location['type'] = app_type.get_access_type(website, app)
                new_location['params'].update(app_type.get_access_params(website, app))
                new_location['path'] = absolute_path(app['path'], website['root'])
                location_info.append(new_location)
            else:
                logging.warn('Skipped unknown location type "%s"', location['type'])

        log_dir = os.path.join(SystemConfig.get(self.context).data['log_dir'], website['name'])
        ensure_directory(
            log_dir,
            uid=SystemConfig.get(self.context).data['nginx']['user'],
            mode=0755
        )

        return Template.by_name(self.context, 'nginx.website.conf').render(data={
            'website': website,
            'location_info': location_info,
        })

    def configure(self):
        cfg = MainConfig.get(self.context)
        system = SystemConfig.get(self.context)
        shutil.rmtree(system.data['nginx']['config_root'])

        ensure_directory(
            system.data['nginx']['config_root'],
            uid=system.data['nginx']['user'],
            mode=0755
        )
        ensure_directory(
            system.data['nginx']['config_vhost_root'],
            uid=system.data['nginx']['user'],
            mode=0755
        )
        ensure_directory(
            system.data['nginx']['config_custom_root'],
            uid=system.data['nginx']['user'],
            mode=0755
        )
        ensure_directory(
            system.data['log_dir'] + '/nginx',
            uid=system.data['nginx']['user'],
            mode=0755
        )

        open(system.data['nginx']['config_file'], 'w').write(
            Template.by_name(self.context, 'nginx.conf').render()
        )
        open(system.data['nginx']['config_file_mime'], 'w').write(
            Template.by_name(self.context, 'nginx.mime.conf').render()
        )
        open(system.data['nginx']['config_file_fastcgi'], 'w').write(
            Template.by_name(self.context, 'nginx.fcgi.conf').render()
        )
        open(system.data['nginx']['config_file_proxy'], 'w').write(
            Template.by_name(self.context, 'nginx.proxy.conf').render()
        )

        for website in cfg.data['websites']:
            if website['enabled']:
                path = os.path.join(system.data['nginx']['config_vhost_root'], website['name'] + '.conf')
                with open(path, 'w') as f:
                    f.write(self.__generate_website_config(website))

        subprocess.call([
            'chown', 'www-data:www-data', '-R', system.data['nginx']['lib_path'],
        ])

        NginxRestartable.get(self.context).schedule_restart()


@component(Restartable)
class NginxRestartable(Restartable):
    name = 'nginx'

    def do_restart(self):
        subprocess.check_call(['service', 'nginx', 'restart'])



from jadi import interface


@interface
class Plugin(object):
    name = None

    @classmethod
    def by_name(cls, context, name):
        for cmd in cls.classes():
            if cmd.name == name:
                return cmd(context)

    def __init__(self, context):
        self.context = context

    def add_config_defaults(self, config):
        pass



from jadi import interface


class CommandArgumentError(Exception):
    pass


@interface
class Command(object):
    name = None
    usage = None

    @classmethod
    def by_name(cls, context, name):
        for cmd in cls.classes():
            if cmd.name == name:
                return cmd(context)

    def __init__(self, context):
        self.context = context

    def consume_arguments(self, argv):
        pass

    def run(self):
        raise NotImplementedError



import logging
from jadi import component

from vvv.api.command import Command, CommandArgumentError
from vvv.api.config import MainConfig


@component(Command)
class WebsiteMaintenanceCommand(Command):
    name = 'maintenance'
    usage = '''
maintenance <website-name> on|off - enables (disables) maintenance mode for a website
'''

    def consume_arguments(self, argv):
        if len(argv) < 1:
            raise CommandArgumentError('short-name is required')
        if len(argv) < 2:
            raise CommandArgumentError('on|off is required')
        self.website_name = argv.pop(0)
        self.on = argv.pop(0) == 'on'

    def run(self):
        cfg = MainConfig.get(self.context)
        for website in cfg.data['websites']:
            if website['name'] == self.website_name:
                website['maintenance_mode'] = self.on
        cfg.save()

        logging.info('Website %s maintenance mode set to: %s', self.website_name, self.on)
